#!/usr/bin/python
# -*- coding: utf-8 -*-

# thumbor imaging service
# https://github.com/thumbor/thumbor/wiki

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license


import thumbor_plugins.optimizers.jp2.config  # noqa

from thumbor.optimizers import BaseOptimizer
from thumbor.utils import logger
from PIL import Image


class Optimizer(BaseOptimizer):
    def __init__(self, context):
        super(Optimizer, self).__init__(context)

        self.jp2_quality = 80
        if self.context.config.JP2_QUALITY:
            if isinstance(self.context.config.JP2_QUALITY, str):
                self.jp2_quality = int(self.context.config.JP2_QUALITY)
            else:
                self.jp2_quality = self.context.config.JP2_QUALITY

    def should_run(self, image_extension, buffer):
        return "jp2" in self.context.request.filters

    def optimize(self, buffer, input_file, output_file):
        im = Image.open(input_file)
        if im.mode in ["1", "P"]:
            if im.mode == "1":
                target_mode = "RGB"
            else:
                # convert() figures out RGB or RGBA based on palette used
                target_mode = None
            im = im.convert(mode=target_mode)

        return im.save(
            output_file,
            "JPEG2000",
            quality_mode="dB",
            quality_layers=[self.jp2_quality],
        )



import os.path

from shutil import which

from tornado.testing import AsyncHTTPTestCase

from thumbor.app import ThumborServiceApp
from thumbor.config import Config
from thumbor.context import Context, ServerParameters
from thumbor.importer import Importer


class EngineCase(AsyncHTTPTestCase):
    def get_app(self):
        cfg = Config(SECURITY_KEY="ACME-SEC")
        server_params = ServerParameters(None, None, None, None, None, None, debug=True)
        server_params.gifsicle_path = which("gifsicle")

        cfg.DETECTORS = []
        cfg.STORAGE = "thumbor.storages.no_storage"
        cfg.LOADER = "thumbor.loaders.file_loader"
        cfg.FILE_LOADER_ROOT_PATH = getattr(self, "fixtures_path", None)
        cfg.ENGINE = "thumbor.engines.pil"
        cfg.USE_GIFSICLE_ENGINE = True
        cfg.FFMPEG_PATH = which("ffmpeg")
        cfg.ENGINE_THREADPOOL_SIZE = 10
        cfg.OPTIMIZERS = [
            getattr(self, "optimizer", None),
        ]
        if not cfg.ENGINE:
            return None

        importer = Importer(cfg)
        importer.import_modules()
        ctx = Context(server_params, cfg, importer)
        application = ThumborServiceApp(ctx)

        return application

    def assert_result_smaller_than_original(self, request, original_image):
        original_image_size = os.path.getsize(
            os.path.join(self.fixtures_path, original_image)
        )
        self.assertLess(int(request.headers["Content-Length"]), original_image_size)



import os.path

from tornado.testing import gen_test

from thumbor_plugins.test_helpers.engine import EngineCase


class PNGQuantTest(EngineCase):
    optimizer = "thumbor_plugins.optimizers.pngquant"
    fixtures_path = os.path.join(os.path.dirname(__file__), "imgs")

    @gen_test
    async def test_pngquant(self):
        result = await self.http_client.fetch(self.get_url("/unsafe/bug.png"))
        self.assert_result_smaller_than_original(result, "bug.png")



#!/usr/bin/python
# -*- coding: utf-8 -*-

# thumbor imaging service
# https://github.com/thumbor/thumbor/wiki

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2011 globo.com thumbor@googlegroups.com

from shutil import which
from unittest import TestCase, mock

from thumbor.config import Config
from thumbor.context import Context, RequestParameters
from thumbor.utils import EXTENSION

from thumbor_plugins.optimizers.jpegrecompress import Optimizer


class MozJpegOptimizerTest(TestCase):
    def setUp(self):
        self.os_path_isfile_patcher = mock.patch("os.path.isfile")
        self.os_access_patcher = mock.patch("os.access")
        self.mock_os_path_isfile = self.os_path_isfile_patcher.start()
        self.mock_os_access = self.os_access_patcher.start()

    def tearDown(self):
        self.os_path_isfile_patcher.stop()
        self.mock_os_access.stop()

    def get_context(self):
        conf = Config()
        conf.JPEGRECOMPRESS_PATH = "/usr/bin/jpeg-recompress"
        ctx = Context(config=conf)
        ctx.request = RequestParameters()

        return ctx

    def test_should_not_run_if_binary_missing(self):
        self.mock_os_path_isfile.return_value = False
        optimizer = Optimizer(self.get_context())
        self.assertFalse(optimizer.should_run(".jpeg", ""))

    def test_should_not_run_if_binary_not_executable(self):
        self.mock_os_access.return_value = False
        optimizer = Optimizer(self.get_context())
        self.assertFalse(optimizer.should_run(".jpeg", ""))

    def test_should_run_only_for_jpg(self):
        optimizer = Optimizer(self.get_context())
        self.assertFalse(optimizer.should_run(".png", ""))

    def test_should_run_for_jpg(self):
        optimizer = Optimizer(self.get_context())
        self.assertTrue(optimizer.should_run(".jpg", ""))

    def test_should_run_for_jpeg(self):
        optimizer = Optimizer(self.get_context())
        self.assertTrue(optimizer.should_run(".jpeg", ""))

    @mock.patch("subprocess.call")
    @mock.patch("PIL.Image.open")
    def test_should_run_mozjpeg_binary(self, pil_image_mock, subprocess_call_mock):
        optimizer = Optimizer(self.get_context())
        optimizer.optimize(None, "input_file", "output_file")
        subprocess_call_mock.assert_called_with(
            "/usr/bin/jpeg-recompress --strip --accurate --loops 10 input_file output_file",
            shell=True,
            stdin=mock.ANY,
        )



#!/usr/bin/python
# -*- coding: utf-8 -*-

# thumbor imaging service
# https://github.com/thumbor/thumbor/wiki

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2011 globo.com thumbor@googlegroups.com

from shutil import which
from unittest import TestCase, mock

from thumbor.config import Config
from thumbor.context import Context, RequestParameters
from thumbor_plugins.optimizers.gifv import Optimizer
from thumbor.utils import EXTENSION


class GifvOptimizerTest(TestCase):
    def setUp(self):
        self.os_path_exists_patcher = mock.patch(
            "thumbor_plugins.optimizers.gifv.exists"
        )
        self.mock_os_path_exists = self.os_path_exists_patcher.start()

    def tearDown(self):
        self.os_path_exists_patcher.stop()

    def get_context(self):
        conf = Config()
        conf.FFMPEG_PATH = "/usr/bin/ffmpeg"
        ctx = Context(config=conf)
        ctx.request = RequestParameters()
        ctx.request.filters.append("gifv")

        return ctx

    def test_should_run_for_gif(self):
        optimizer = Optimizer(self.get_context())
        self.assertTrue(optimizer.should_run(".gif", ""))

    def test_should_not_run_for_not_gif(self):
        optimizer = Optimizer(self.get_context())
        for ext in EXTENSION.items():
            if ext != ".gif":
                self.assertFalse(optimizer.should_run(ext, ""))

    def test_should_not_run_if_binary_ffmpeg_path_does_not_exist(self):
        self.mock_os_path_exists.return_value = False

        optimizer = Optimizer(self.get_context())
        self.assertFalse(optimizer.should_run(".gif", ""))

    def test_should_parse_background_color_with_valid_value(self):
        optimizer = Optimizer(self.get_context())
        magenta_unicode_hex = "#ff00ff"
        self.assertEqual(
            optimizer.normalize_color_to_hex("ff00ff"), magenta_unicode_hex
        )
        self.assertEqual(
            optimizer.normalize_color_to_hex("#ff00ff"), magenta_unicode_hex
        )
        self.assertEqual(optimizer.normalize_color_to_hex("f0f"), magenta_unicode_hex)
        self.assertEqual(optimizer.normalize_color_to_hex("#f0f"), magenta_unicode_hex)
        self.assertEqual(
            optimizer.normalize_color_to_hex("magenta"), magenta_unicode_hex
        )

    def test_should_not_parse_background_color_with_invalid_value(self):
        optimizer = Optimizer(self.get_context())
        self.assertEqual(optimizer.normalize_color_to_hex("asdfasdfasfd"), None)



from thumbor.config import Config

Config.define(
    "MOZJPEG_PATH",
    "/opt/mozjpeg/bin/cjpeg",
    "Path for the mozjpeg binary",
    "Optimizers",
)

Config.define(
    "MOZJPEG_QUALITY", "75", "Optimization level for mozjpeg (0-100)", "Optimizers"
)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# thumbor imaging service
# https://github.com/thumbor/thumbor/wiki

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license


import os
import subprocess

import thumbor_plugins.optimizers.pngquant.config  # noqa

from thumbor.optimizers import BaseOptimizer
from thumbor.utils import logger


class Optimizer(BaseOptimizer):
    def __init__(self, context):
        super(Optimizer, self).__init__(context)

        self.runnable = True
        self.pngquant_path = self.context.config.PNGQUANT_PATH
        self.pngquant_quality = self.context.config.PNGQUANT_QUALITY or "65-80"
        self.pngquant_speed = self.context.config.PNGQUANT_SPEED or "3"

        if not (
            os.path.isfile(self.pngquant_path)
            and os.access(self.pngquant_path, os.X_OK)
        ):
            logger.error(
                "ERROR pnqquant path '{0}' is not accessible".format(self.pngquant_path)
            )
            self.runnable = False

    def should_run(self, image_extension, buffer):
        return "png" in image_extension and self.runnable

    def optimize(self, buffer, input_file, output_file):
        command = "cat %s | %s --speed %s --quality=%s - > %s" % (
            input_file,
            self.pngquant_path,
            self.pngquant_speed,
            self.pngquant_quality,
            output_file,
        )
        with open(os.devnull) as null:
            logger.debug("[PNGQUANT] running: " + command)
            subprocess.call(command, shell=True, stdin=null)



from thumbor.config import Config

Config.define(
    "PNGQUANT_PATH", "/usr/bin/pngquant", "Path for the pngquant binary", "Optimizers"
)

Config.define(
    "PNGQUANT_QUALITY",
    "65-80",
    "Optimization level for pngquant ([0..100]-[0..100])",
    "Optimizers",
)

Config.define(
    "PNGQUANT_SPEED",
    "1",
    "Optimization speed for pngquant 1-11, 1 is slowest",
    "Optimizers",
)



from thumbor.config import Config

Config.define(
    "OPTIPNG_PATH", "/usr/bin/optipng", "Path for the optipng binary", "Optimizers"
)

Config.define("OPTIPNG_LEVEL", 5, "Optimization level for optipng (0-7)", "Optimizers")



#!/usr/bin/python
# -*- coding: utf-8 -*-

from setuptools import setup



setup(
    name="thumbor-plugins-jpegrecompress",
    version="0.1.3",
    keywords="thumbor optimizers jpegrecompress",
    author="Guilherme Souza",
    author_email="guilherme@souza.tech",
    url="https://github.com/thumbor/thumbor-plugins",
    license="MIT",
    description="Thumbor optimizer to run jpegrecompress",
    long_description="""
        Available Configs:
        - JPEGRECOMPRESS_PATH

        This package is part of the thumbor-plugins repository.
        For more information, visit: https://github.com/thumbor/thumbor-plugins
    """,
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Operating System :: MacOS",
        "Operating System :: POSIX :: Linux",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3 :: Only",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
        "Topic :: Multimedia :: Graphics :: Presentation",
    ],
    packages=[
        "thumbor_plugins.optimizers.jpegrecompress",
    ],
    package_dir={
        "thumbor_plugins.optimizers.jpegrecompress": ".",
    },
)



import os.path

from tornado.testing import gen_test

from thumbor_plugins.test_helpers.engine import EngineCase


class GIFVTest(EngineCase):
    optimizer = "thumbor_plugins.optimizers.gifv"
    fixtures_path = os.path.join(os.path.dirname(__file__), "imgs")

    @gen_test
    async def test_gifv(self):
        response = await self.http_client.fetch(
            self.get_url("/unsafe/filters:gifv()/animated.gif")
        )
        self.assertEqual(response.code, 200)
        self.assertEqual(response.headers["Content-Type"], "video/mp4")

    @gen_test
    async def test_gifv_with_background(self):
        response = await self.http_client.fetch(
            self.get_url("/unsafe/filters:gifv(mp4):background_color(ff00ff)/animated.gif")
        )
        self.assertEqual(response.code, 200)
        self.assertEqual(response.headers["Content-Type"], "video/mp4")

    @gen_test
    async def test_gifv_webm(self):
        response = await self.http_client.fetch(
            self.get_url("/unsafe/filters:gifv(webm)/animated.gif")
        )
        self.assertEqual(response.code, 200)
        self.assertEqual(response.headers["Content-Type"], "video/webm")



#!/usr/bin/python
# -*- coding: utf-8 -*-

# thumbor imaging service
# https://github.com/thumbor/thumbor/wiki

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2011 globo.com thumbor@googlegroups.com

from shutil import which
from unittest import TestCase, mock

from thumbor.config import Config
from thumbor.context import Context, RequestParameters
from thumbor.utils import EXTENSION

from thumbor_plugins.optimizers.jp2 import Optimizer


class Jp2OptimizerTest(TestCase):
    def get_context(self):
        conf = Config()
        conf.JP2_QUALITY = "80"
        ctx = Context(config=conf)
        ctx.request = RequestParameters()
        ctx.request.filters.append("jp2")

        return ctx

    @mock.patch("thumbor_plugins.optimizers.jp2.Image")
    def test_should_convert_image_as_rgb(self, pil_image_mock):
        image_opened_mock = mock.Mock()
        image_converted_mock = mock.Mock()
        image_opened_mock.convert.return_value = image_converted_mock
        pil_image_mock.open.return_value = image_opened_mock

        image_opened_mock.mode = "1"

        optimizer = Optimizer(self.get_context())
        optimizer.optimize(None, "input_file", "output_file")

        image_opened_mock.convert.assert_called_with(mode="RGB")
        image_converted_mock.save.assert_called_with(
            "output_file", "JPEG2000", quality_mode="dB", quality_layers=[80]
        )
